/**
 * Copyright (c) 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.api.command;

import java.util.Iterator;
import java.util.ServiceLoader;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.ow2.petals.cli.api.shell.Shell;
import org.ow2.petals.cli.api.shell.exception.ConnectionCommandAlreadyRegisteredException;
import org.ow2.petals.cli.api.shell.exception.DisconnectionCommandAlreadyRegisteredException;
import org.ow2.petals.cli.api.shell.exception.DuplicatedCommandException;

public class CommandRegistration {

    private static final Logger LOG = Logger.getLogger(CommandRegistration.class.getName());

    private CommandRegistration() {
        // Reduce visibility of the constructor because it's an utility class
    }

    /**
     * Registers commands declared on the classpath into the given shell. Commands are declared through the service
     * provider {@link Command}.
     * 
     * @param shell
     *            The shell in which commands will be registered.
     */
    public static void registers(final Shell shell) {
        final ServiceLoader<Command> commandLoader = ServiceLoader.load(Command.class);
        final Iterator<Command> commandsIterator = commandLoader.iterator();
        while (commandsIterator.hasNext()) {
            final Command command = commandsIterator.next();
            try {
                command.setShell(shell);

                if (command instanceof ConnectionCommand) {
                    shell.registersConnectionCommand((ConnectionCommand) command);
                    LOG.fine(String.format("Connection command '%s' registered.", command.getName()));
                } else if (command instanceof DisconnectionCommand) {
                    shell.registersDisconnectionCommand((DisconnectionCommand) command);
                    LOG.fine(String.format("Disconnection command '%s' registered.", command.getName()));
                } else {
                    shell.registersCommand(command);
                    LOG.fine(String.format("Command '%s' registered.", command.getName()));
                }
            } catch (final DuplicatedCommandException e) {
                // Command already registered
                LOG.log(Level.SEVERE,
                        "Trying to register a command already registered ('" + command.getName() + "'). Ignored.", e);
            } catch (final ConnectionCommandAlreadyRegisteredException e) {
                // A connection command was already registered
                LOG.log(Level.SEVERE,
                        "Trying to register another connection command: '" + command.getName() + "'. Ignored.", e);
            } catch (final DisconnectionCommandAlreadyRegisteredException e) {
                // A disconnection command was already registered
                LOG.log(Level.SEVERE,
                        "Trying to register another disconnection command: '" + command.getName() + "'. Ignored.", e);
            } catch (final IllegalArgumentException e) {
                // It's a bug
                LOG.log(Level.SEVERE, "Trying to register a null command. Ignored.", e);
            }
        }
    }

}
